'''OpenGL extension NV.uniform_buffer_std430_layout

This module customises the behaviour of the 
OpenGL.raw.GL.NV.uniform_buffer_std430_layout to provide a more 
Python-friendly API

Overview (from the spec)
	
	OpenGL 4.3 (and ARB_enhanced_layouts) provide an enhanced layout
	qualifier syntax for aligning members of uniform and shader storage
	blocks.  The std430 enhanced layout qualifier is advantageous,
	compared with std140, because it provides a more space-efficient
	layout of arrays that more easily matches the data layout in C/C++
	structures stored in CPU memory.
	
	However OpenGL 4.3 precluded using the std430 layout qualifier for
	uniform blocks (by mandating a compilation error be generated).
	
	This extension makes std430 a legal layout qualifier for uniform
	blocks in GLSL when the extension's GLSL #extension functionality
	is enabled or required.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/uniform_buffer_std430_layout.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.uniform_buffer_std430_layout import *
from OpenGL.raw.GL.NV.uniform_buffer_std430_layout import _EXTENSION_NAME

def glInitUniformBufferStd430LayoutNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_bindless_texture'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_bindless_texture',error_checker=_errors._error_checker)

@_f
@_p.types(_cs.GLuint64,_cs.GLuint,_cs.GLint,_cs.GLboolean,_cs.GLint,_cs.GLenum)
def glGetImageHandleNV(texture,level,layered,layer,format):pass
@_f
@_p.types(_cs.GLuint64,_cs.GLuint)
def glGetTextureHandleNV(texture):pass
@_f
@_p.types(_cs.GLuint64,_cs.GLuint,_cs.GLuint)
def glGetTextureSamplerHandleNV(texture,sampler):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint64)
def glIsImageHandleResidentNV(handle):pass
@_f
@_p.types(_cs.GLboolean,_cs.GLuint64)
def glIsTextureHandleResidentNV(handle):pass
@_f
@_p.types(None,_cs.GLuint64)
def glMakeImageHandleNonResidentNV(handle):pass
@_f
@_p.types(None,_cs.GLuint64,_cs.GLenum)
def glMakeImageHandleResidentNV(handle,access):pass
@_f
@_p.types(None,_cs.GLuint64)
def glMakeTextureHandleNonResidentNV(handle):pass
@_f
@_p.types(None,_cs.GLuint64)
def glMakeTextureHandleResidentNV(handle):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLuint64)
def glProgramUniformHandleui64NV(program,location,value):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glProgramUniformHandleui64vNV(program,location,count,values):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLuint64)
def glUniformHandleui64NV(location,value):pass
@_f
@_p.types(None,_cs.GLint,_cs.GLsizei,arrays.GLuint64Array)
def glUniformHandleui64vNV(location,count,value):pass



'''OpenGL extension EXT.separate_depth_stencil

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.separate_depth_stencil to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds support for binding a different image to the depth and
	stencil attachments of a framebuffer object.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/separate_depth_stencil.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.separate_depth_stencil import *
from OpenGL.raw.GLES2.EXT.separate_depth_stencil import _EXTENSION_NAME

def glInitSeparateDepthStencilEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.texture_storage_multisample_2d_array

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.texture_storage_multisample_2d_array to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension provides support for a new type of immutable texture,
	two-dimensional multisample array textures. It depends on functionality
	introduced in OpenGL ES 3.1 to support two-dimensional multisample
	(non-array) textures.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/texture_storage_multisample_2d_array.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.texture_storage_multisample_2d_array import *
from OpenGL.raw.GLES2.OES.texture_storage_multisample_2d_array import _EXTENSION_NAME

def glInitTextureStorageMultisample2DArrayOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension OES.texture_float_linear

This module customises the behaviour of the 
OpenGL.raw.GLES2.OES.texture_float_linear to provide a more 
Python-friendly API

Overview (from the spec)
	
	These extensions expand upon the OES_texture_half_float and
	OES_texture_float extensions by allowing support for LINEAR
	magnification filter and LINEAR, NEAREST_MIPMAP_LINEAR,
	LINEAR_MIPMAP_NEAREST and LINEAR_MIPMAP_NEAREST minification
	filters.
	
	When implemented against OpenGL ES 3.0 or later versions, the
	existing sized 32-bit floating-point formats become texture-filterable,
	but no new formats are added.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/texture_float_linear.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.OES.texture_float_linear import *
from OpenGL.raw.GLES2.OES.texture_float_linear import _EXTENSION_NAME

def glInitTextureFloatLinearOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_SGIX_resample'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_SGIX_resample',error_checker=_errors._error_checker)
GL_PACK_RESAMPLE_SGIX=_C('GL_PACK_RESAMPLE_SGIX',0x842E)
GL_RESAMPLE_DECIMATE_SGIX=_C('GL_RESAMPLE_DECIMATE_SGIX',0x8430)
GL_RESAMPLE_REPLICATE_SGIX=_C('GL_RESAMPLE_REPLICATE_SGIX',0x8433)
GL_RESAMPLE_ZERO_FILL_SGIX=_C('GL_RESAMPLE_ZERO_FILL_SGIX',0x8434)
GL_UNPACK_RESAMPLE_SGIX=_C('GL_UNPACK_RESAMPLE_SGIX',0x842F)




"""Numbers passed as array handling code for PyOpenGL
"""

REGISTRY_NAME = 'numbers'
from OpenGL.raw.GL import _types
from OpenGL.raw.GL.VERSION import GL_1_1
from OpenGL.arrays import formathandler
import ctypes
from OpenGL._bytes import long, integer_types

n = c = None


class NumberHandler(formathandler.FormatHandler):
    """Allows the user to pass a bald Python float,int, etceteras as an array-of-1"""

    HANDLED_TYPES = integer_types + (
        float,
        _types.GLdouble,
        _types.GLfloat,
        _types.GLint,
        _types.GLshort,
        _types.GLuint,
        _types.GLulong,
        _types.GLushort,
        _types.GLclampf,
        _types.GLclampd,
    )

    def from_param(self, value, typeCode=None):
        """If it's a ctypes value, pass on, otherwise do asArray"""
        try:
            return ctypes.byref(value)
        except TypeError as err:
            err.args += (
                ' If you have ERROR_ON_COPY enabled, remember to pass in an array to array-requiring functions.',
            )
            raise

    dataPointer = from_param

    def zeros(self, dims, typeCode=None):
        """Currently don't allow Number as output types!"""
        raise NotImplementedError(
            """Number data-type not allowed as an output array format"""
        )

    def ones(self, dims, typeCode=None):
        """Currently don't allow Number as output types!"""
        raise NotImplementedError(
            """Number data-type not allowed as an output array format"""
        )

    def arrayToGLType(self, value):
        """Given a value, guess OpenGL type of the corresponding pointer"""
        if value.__class__ in TARGET_TYPES:
            return TARGET_TYPES[value.__class__]
        else:
            guess = DEFAULT_TYPES.get(value.__class__)
            if guess is not None:
                return guess[1]
            raise TypeError(
                """Can't guess array data-type for %r types""" % (type(value))
            )

    def arraySize(self, value, typeCode=None):
        """Given a data-value, calculate ravelled size for the array"""
        return 1

    def asArray(self, value, typeCode=None):
        """Convert given value to an array value of given typeCode"""

        if value.__class__ in TARGET_TYPES:
            return value
        targetType = CONSTANT_TO_TYPE.get(typeCode)
        if targetType is not None:
            return targetType(value)
        raise TypeError(
            """Don't know how to convert %r to an array type""" % (typeCode,)
        )

    def unitSize(self, value, typeCode=None):
        """Determine unit size of an array (if possible)"""
        return 1  # there's only 1 possible value in the set...

    def registerEquivalent(self, typ, base):
        """Register a sub-class for handling as the base-type"""
        global TARGET_TYPE_TUPLE
        for source in (DEFAULT_TYPES, TARGET_TYPES, BYTE_SIZES):
            if base in source:
                source[typ] = source[base]
        if base in TARGET_TYPES:
            TARGET_TYPE_TUPLE = TARGET_TYPE_TUPLE + (base,)


DEFAULT_TYPES = {
    float: (_types.GLdouble, GL_1_1.GL_DOUBLE),
    int: (_types.GLint, GL_1_1.GL_INT),
    long: (_types.GLint, GL_1_1.GL_INT),
}
TARGET_TYPES = dict(
    [(getattr(_types, n), c) for (n, c) in _types.ARRAY_TYPE_TO_CONSTANT]
)
TARGET_TYPE_TUPLE = tuple(
    [getattr(_types, n) for (n, c) in _types.ARRAY_TYPE_TO_CONSTANT]
)
CONSTANT_TO_TYPE = dict(
    [(c, getattr(_types, n)) for (n, c) in _types.ARRAY_TYPE_TO_CONSTANT]
)

BYTE_SIZES = dict(
    [(c, ctypes.sizeof(getattr(_types, n))) for (n, c) in _types.ARRAY_TYPE_TO_CONSTANT]
)

try:
    del n, c
except NameError as err:
    pass



'''OpenGL extension IMG.texture_env_enhanced_fixed_function

This module customises the behaviour of the 
OpenGL.raw.GLES1.IMG.texture_env_enhanced_fixed_function to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds new texture environment functions to allow use of 
	of blend modes supported in some early MBX-lite devices, including dot3 functionality. 
	It is superceded by OpenGL-ES 1.1 which includes tex_env_combine.
	
	New functions may be specified by calling TexEnv with the following tokens: 
	MODULATE_COLOR_IMG, RECIP_ADD_SIGNED_ALPHA_IMG, TEXTURE_ALPHA_MODULATE_IMG, 
	FACTOR_ALPHA_MODULATE_IMG, FRAGMENT_ALPHA_MODULATE_IMG, ADD_BLEND_IMG, DOT3_RGBA.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/IMG/texture_env_enhanced_fixed_function.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.IMG.texture_env_enhanced_fixed_function import *
from OpenGL.raw.GLES1.IMG.texture_env_enhanced_fixed_function import _EXTENSION_NAME

def glInitTextureEnvEnhancedFixedFunctionIMG():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_EXT_discard_framebuffer'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_EXT_discard_framebuffer',error_checker=_errors._error_checker)
GL_COLOR_EXT=_C('GL_COLOR_EXT',0x1800)
GL_DEPTH_EXT=_C('GL_DEPTH_EXT',0x1801)
GL_STENCIL_EXT=_C('GL_STENCIL_EXT',0x1802)
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,arrays.GLuintArray)
def glDiscardFramebufferEXT(target,numAttachments,attachments):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_OES_EGL_image_external'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_OES_EGL_image_external',error_checker=_errors._error_checker)
GL_REQUIRED_TEXTURE_IMAGE_UNITS_OES=_C('GL_REQUIRED_TEXTURE_IMAGE_UNITS_OES',0x8D68)
GL_SAMPLER_EXTERNAL_OES=_C('GL_SAMPLER_EXTERNAL_OES',0x8D66)
GL_TEXTURE_BINDING_EXTERNAL_OES=_C('GL_TEXTURE_BINDING_EXTERNAL_OES',0x8D67)
GL_TEXTURE_EXTERNAL_OES=_C('GL_TEXTURE_EXTERNAL_OES',0x8D65)




"""String-array-handling code for PyOpenGL
"""

from OpenGL.raw.GL import _types
from OpenGL.raw.GL.VERSION import GL_1_1
from OpenGL.arrays import formathandler
import ctypes
from OpenGL import _bytes, error
from OpenGL._configflags import ERROR_ON_COPY


def dataPointer(value, typeCode=None):
    return ctypes.cast(ctypes.c_char_p(value), ctypes.c_void_p).value


class StringHandler(formathandler.FormatHandler):
    """String-specific data-type handler for OpenGL"""

    HANDLED_TYPES = (_bytes.bytes,)

    @classmethod
    def from_param(cls, value, typeCode=None):
        return ctypes.c_void_p(dataPointer(value))

    dataPointer = staticmethod(dataPointer)

    def zeros(self, dims, typeCode=None):
        """Currently don't allow strings as output types!"""
        raise NotImplementedError(
            """Don't currently support strings as output arrays"""
        )

    def ones(self, dims, typeCode=None):
        """Currently don't allow strings as output types!"""
        raise NotImplementedError(
            """Don't currently support strings as output arrays"""
        )

    def arrayToGLType(self, value):
        """Given a value, guess OpenGL type of the corresponding pointer"""
        raise NotImplementedError(
            """Can't guess data-type from a string-type argument"""
        )

    def arraySize(self, value, typeCode=None):
        """Given a data-value, calculate ravelled size for the array"""
        # need to get bits-per-element...
        byteCount = BYTE_SIZES[typeCode]
        return len(value) // byteCount

    def arrayByteCount(self, value, typeCode=None):
        """Given a data-value, calculate number of bytes required to represent"""
        return len(value)

    def asArray(self, value, typeCode=None):
        """Convert given value to an array value of given typeCode"""
        if isinstance(value, bytes):
            return value
        elif hasattr(value, 'tostring'):
            return value.tostring()
        elif hasattr(value, 'raw'):
            return value.raw
        # could convert types to string here, but we're not registered for
        # anything save string types...
        raise TypeError("""String handler got non-string object: %r""" % (type(value)))

    def dimensions(self, value, typeCode=None):
        """Determine dimensions of the passed array value (if possible)"""
        raise TypeError("""Cannot calculate dimensions for a String data-type""")


class UnicodeHandler(StringHandler):
    HANDLED_TYPES = (_bytes.unicode,)

    @classmethod
    def from_param(cls, value, typeCode=None):
        # TODO: raise CopyError if the flag is set!
        converted = _bytes.as_8_bit(value)
        result = StringHandler.from_param(converted)
        if converted is not value:
            if ERROR_ON_COPY:
                raise error.CopyError(
                    """Unicode string passed, cannot copy with ERROR_ON_COPY set, please use 8-bit strings"""
                )
            result._temporary_array_ = converted
        return result

    def asArray(self, value, typeCode=None):
        value = _bytes.as_8_bit(value)
        return StringHandler.asArray(self, value, typeCode=typeCode)


BYTE_SIZES = {
    GL_1_1.GL_DOUBLE: ctypes.sizeof(_types.GLdouble),
    GL_1_1.GL_FLOAT: ctypes.sizeof(_types.GLfloat),
    GL_1_1.GL_INT: ctypes.sizeof(_types.GLint),
    GL_1_1.GL_SHORT: ctypes.sizeof(_types.GLshort),
    GL_1_1.GL_UNSIGNED_BYTE: ctypes.sizeof(_types.GLubyte),
    GL_1_1.GL_UNSIGNED_SHORT: ctypes.sizeof(_types.GLshort),
    GL_1_1.GL_BYTE: ctypes.sizeof(_types.GLbyte),
    GL_1_1.GL_UNSIGNED_INT: ctypes.sizeof(_types.GLuint),
}



'''OpenGL extension NV.texture_compression_s3tc_update

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.texture_compression_s3tc_update to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension allows for full or partial image updates to a
	compressed 2D texture from an uncompressed texel data buffer using
	TexImage2D and TexSubImage2D. Consquently, if a compressed internal
	format is used, all the restrictions associated with compressed
	textures will apply. These include sub-image updates aligned to 4x4
	pixel blocks and the restriction on usage as render targets.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/texture_compression_s3tc_update.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.texture_compression_s3tc_update import *
from OpenGL.raw.GLES2.NV.texture_compression_s3tc_update import _EXTENSION_NAME

def glInitTextureCompressionS3TcUpdateNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_QCOM_motion_estimation'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_QCOM_motion_estimation',error_checker=_errors._error_checker)
GL_FOVEATION_SCALED_BIN_METHOD_BIT_QCOM=_C('GL_FOVEATION_SCALED_BIN_METHOD_BIT_QCOM',0x00000002)
GL_MOTION_ESTIMATION_SEARCH_BLOCK_X_QCOM=_C('GL_MOTION_ESTIMATION_SEARCH_BLOCK_X_QCOM',0x8C90)
GL_MOTION_ESTIMATION_SEARCH_BLOCK_Y_QCOM=_C('GL_MOTION_ESTIMATION_SEARCH_BLOCK_Y_QCOM',0x8C91)
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glTexEstimateMotionQCOM(ref,target,output):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLuint,_cs.GLuint,_cs.GLuint)
def glTexEstimateMotionRegionsQCOM(ref,target,output,mask):pass



"""Wrapper/Implementation of the GLU tessellator objects for PyOpenGL"""
from OpenGL.raw import GLU as _simple
from OpenGL.raw.GL.VERSION import GL_1_1
from OpenGL.platform import createBaseFunction
from OpenGL.GLU import glustruct
from OpenGL import arrays, wrapper
from OpenGL.platform import PLATFORM

GLU = PLATFORM.GLU
from OpenGL.lazywrapper import lazy as _lazy
import ctypes


class GLUtesselator(glustruct.GLUStruct, _simple.GLUtesselator):
    """Implementation class for GLUTessellator structures in OpenGL-ctypes"""

    FUNCTION_TYPE = PLATFORM.functionTypeFor(PLATFORM.GLU)
    CALLBACK_TYPES = {
        # mapping from "which" GLU enumeration to a ctypes function type
        _simple.GLU_TESS_BEGIN: FUNCTION_TYPE(None, _simple.GLenum),
        _simple.GLU_TESS_BEGIN_DATA: FUNCTION_TYPE(
            None, _simple.GLenum, ctypes.c_void_p
        ),
        _simple.GLU_TESS_EDGE_FLAG: FUNCTION_TYPE(None, _simple.GLboolean),
        _simple.GLU_TESS_EDGE_FLAG_DATA: FUNCTION_TYPE(
            None, _simple.GLboolean, ctypes.c_void_p
        ),
        _simple.GLU_TESS_VERTEX: FUNCTION_TYPE(None, ctypes.c_void_p),
        _simple.GLU_TESS_VERTEX_DATA: FUNCTION_TYPE(
            None, ctypes.c_void_p, ctypes.c_void_p
        ),
        _simple.GLU_TESS_END: FUNCTION_TYPE(None),
        _simple.GLU_TESS_END_DATA: FUNCTION_TYPE(None, ctypes.c_void_p),
        _simple.GLU_TESS_COMBINE: FUNCTION_TYPE(
            None,
            ctypes.POINTER(_simple.GLdouble),
            ctypes.POINTER(ctypes.c_void_p),
            ctypes.POINTER(_simple.GLfloat),
            ctypes.POINTER(ctypes.c_void_p),
        ),
        _simple.GLU_TESS_COMBINE_DATA: FUNCTION_TYPE(
            None,
            ctypes.POINTER(_simple.GLdouble),
            ctypes.POINTER(ctypes.c_void_p),
            ctypes.POINTER(_simple.GLfloat),
            ctypes.POINTER(ctypes.c_void_p),
            ctypes.c_void_p,
        ),
        _simple.GLU_TESS_ERROR: FUNCTION_TYPE(None, _simple.GLenum),
        _simple.GLU_TESS_ERROR_DATA: FUNCTION_TYPE(
            None, _simple.GLenum, ctypes.c_void_p
        ),
        _simple.GLU_ERROR: FUNCTION_TYPE(None, _simple.GLenum),
    }
    WRAPPER_METHODS = {
        _simple.GLU_TESS_BEGIN_DATA: 'dataWrapper',
        _simple.GLU_TESS_EDGE_FLAG_DATA: 'dataWrapper',
        _simple.GLU_TESS_VERTEX: 'vertexWrapper',
        _simple.GLU_TESS_VERTEX_DATA: 'vertexWrapper',
        _simple.GLU_TESS_END_DATA: 'dataWrapper',
        _simple.GLU_TESS_COMBINE: 'combineWrapper',
        _simple.GLU_TESS_COMBINE_DATA: 'combineWrapper',
        _simple.GLU_TESS_ERROR_DATA: 'dataWrapper',
    }

    def gluTessVertex(self, location, data=None):
        """Add a vertex to this tessellator, storing data for later lookup"""
        vertexCache = getattr(self, 'vertexCache', None)
        if vertexCache is None:
            self.vertexCache = []
            vertexCache = self.vertexCache
        location = arrays.GLdoubleArray.asArray(location, GL_1_1.GL_DOUBLE)
        if arrays.GLdoubleArray.arraySize(location) != 3:
            raise ValueError(
                """Require 3 doubles for array location, got: %s""" % (location,)
            )
        oorValue = self.noteObject(data)
        vp = ctypes.c_void_p(oorValue)
        self.vertexCache.append(location)
        return gluTessVertexBase(self, location, vp)

    def gluTessBeginPolygon(self, data):
        """Note the object pointer to return it as a Python object"""
        return _simple.gluTessBeginPolygon(self, ctypes.c_void_p(self.noteObject(data)))

    def combineWrapper(self, function):
        """Wrap a Python function with ctypes-compatible wrapper for combine callback

        For a Python combine callback, the signature looks like this:
            def combine(
                GLdouble coords[3],
                void *vertex_data[4],
                GLfloat weight[4]
            ):
                return data
        While the C signature looks like this:
            void combine(
                GLdouble coords[3],
                void *vertex_data[4],
                GLfloat weight[4],
                void **outData
            )
        """
        if (function is not None) and (not hasattr(function, '__call__')):
            raise TypeError("""Require a callable callback, got:  %s""" % (function,))

        def wrap(coords, vertex_data, weight, outData, *args):
            """The run-time wrapper around the function"""
            coords = self.ptrAsArray(coords, 3, arrays.GLdoubleArray)
            weight = self.ptrAsArray(weight, 4, arrays.GLfloatArray)
            # find the original python objects for vertex data
            vertex_data = [self.originalObject(vertex_data[i]) for i in range(4)]
            args = tuple([self.originalObject(x) for x in args])
            try:
                result = function(coords, vertex_data, weight, *args)
            except Exception as err:
                raise err.__class__(
                    """Failure during combine callback %r with args( %s,%s,%s,*%s):\n%s"""
                    % (
                        function,
                        coords,
                        vertex_data,
                        weight,
                        args,
                        str(err),
                    )
                )
            outP = ctypes.c_void_p(self.noteObject(result))
            if outData:
                outData[0] = outP
            else:
                raise RuntimeError("Null outData passed to callback")
            return None

        return wrap

    def dataWrapper(self, function):
        """Wrap a function which only has the one data-pointer as last arg"""
        if (function is not None) and (not hasattr(function, '__call__')):
            raise TypeError("""Require a callable callback, got:  %s""" % (function,))

        def wrap(*args):
            """Just return the original object for polygon_data"""
            args = args[:-1] + (self.originalObject(args[-1]),)
            try:
                return function(*args)
            except Exception as err:
                err.args += (function, args)
                raise

        return wrap

    def dataWrapper2(self, function):
        """Wrap a function which has two data-pointers as last args"""
        if (function is not None) and (not hasattr(function, '__call__')):
            raise TypeError("""Require a callable callback, got:  %s""" % (function,))

        def wrap(*args):
            """Just return the original object for polygon_data"""
            args = args[:-2] + (
                self.originalObject(args[-2]),
                self.originalObject(args[-1]),
            )
            try:
                return function(*args)
            except Exception as err:
                err.args += (function, args)
                raise

        return wrap

    def vertexWrapper(self, function):
        """Converts a vertex-pointer into an OOR vertex for processing"""
        if (function is not None) and (not hasattr(function, '__call__')):
            raise TypeError("""Require a callable callback, got:  %s""" % (function,))

        def wrap(vertex, data=None):
            """Just return the original object for polygon_data"""
            vertex = self.originalObject(vertex)
            try:
                if data is not None:
                    data = self.originalObject(data)
                    return function(vertex, data)
                else:
                    return function(vertex)
            except Exception as err:
                err.args += (function, (vertex, data))
                raise

        return wrap


c = funcType = None
GLUtesselator.CALLBACK_FUNCTION_REGISTRARS = dict(
    [
        (
            c,
            createBaseFunction(
                'gluTessCallback',
                dll=GLU,
                resultType=None,
                argTypes=[ctypes.POINTER(GLUtesselator), _simple.GLenum, funcType],
                doc='gluTessCallback( POINTER(GLUtesselator)(tess), GLenum(which), _GLUfuncptr(CallBackFunc) ) -> None',
                argNames=('tess', 'which', 'CallBackFunc'),
            ),
        )
        for (c, funcType) in GLUtesselator.CALLBACK_TYPES.items()
    ]
)
try:
    del c, funcType
except NameError as err:
    pass


def gluTessCallback(tess, which, function):
    """Set a given gluTessellator callback for the given tessellator"""
    return tess.addCallback(which, function)


def gluTessBeginPolygon(tess, data):
    """Start definition of polygon in the tessellator"""
    return tess.gluTessBeginPolygon(data)


def gluTessVertex(tess, location, data=None):
    """Add a vertex to the tessellator's current polygon"""
    return tess.gluTessVertex(location, data)


# /usr/include/GL/glu.h 293
@_lazy(
    createBaseFunction(
        'gluNewTess',
        dll=GLU,
        resultType=ctypes.POINTER(GLUtesselator),
        doc='gluNewTess(  ) -> POINTER(GLUtesselator)',
    )
)
def gluNewTess(baseFunction):
    """Get a new tessellator object (just unpacks the pointer for you)"""
    return baseFunction()[0]


@_lazy(_simple.gluGetTessProperty)
def gluGetTessProperty(baseFunction, tess, which, data=None):
    """Retrieve single double for a tessellator property"""
    if data is None:
        data = _simple.GLdouble(0.0)
        baseFunction(tess, which, data)
        return data.value
    else:
        return baseFunction(tess, which, data)


gluTessVertexBase = wrapper.wrapper(_simple.gluTessVertex).setInputArraySize(
    'location',
    3,
)

__all__ = (
    'gluNewTess',
    'gluGetTessProperty',
    'gluTessBeginPolygon',
    'gluTessCallback',
    'gluTessVertex',
)



'''OpenGL extension OES.element_index_uint

This module customises the behaviour of the 
OpenGL.raw.GLES1.OES.element_index_uint to provide a more 
Python-friendly API

Overview (from the spec)
	
	OpenGL ES 1.0 supports DrawElements with <type> value of
	UNSIGNED_BYTE and UNSIGNED_SHORT.  This extension adds
	support for UNSIGNED_INT <type> values.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/OES/element_index_uint.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES1 import _types, _glgets
from OpenGL.raw.GLES1.OES.element_index_uint import *
from OpenGL.raw.GLES1.OES.element_index_uint import _EXTENSION_NAME

def glInitElementIndexUintOES():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_EXT_texture_storage_compression'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_EXT_texture_storage_compression',error_checker=_errors._error_checker)
GL_NUM_SURFACE_COMPRESSION_FIXED_RATES_EXT=_C('GL_NUM_SURFACE_COMPRESSION_FIXED_RATES_EXT',0x8F6E)
GL_SURFACE_COMPRESSION_EXT=_C('GL_SURFACE_COMPRESSION_EXT',0x96C0)
GL_SURFACE_COMPRESSION_FIXED_RATE_10BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_10BPC_EXT',0x96CD)
GL_SURFACE_COMPRESSION_FIXED_RATE_11BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_11BPC_EXT',0x96CE)
GL_SURFACE_COMPRESSION_FIXED_RATE_12BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_12BPC_EXT',0x96CF)
GL_SURFACE_COMPRESSION_FIXED_RATE_1BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_1BPC_EXT',0x96C4)
GL_SURFACE_COMPRESSION_FIXED_RATE_2BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_2BPC_EXT',0x96C5)
GL_SURFACE_COMPRESSION_FIXED_RATE_3BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_3BPC_EXT',0x96C6)
GL_SURFACE_COMPRESSION_FIXED_RATE_4BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_4BPC_EXT',0x96C7)
GL_SURFACE_COMPRESSION_FIXED_RATE_5BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_5BPC_EXT',0x96C8)
GL_SURFACE_COMPRESSION_FIXED_RATE_6BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_6BPC_EXT',0x96C9)
GL_SURFACE_COMPRESSION_FIXED_RATE_7BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_7BPC_EXT',0x96CA)
GL_SURFACE_COMPRESSION_FIXED_RATE_8BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_8BPC_EXT',0x96CB)
GL_SURFACE_COMPRESSION_FIXED_RATE_9BPC_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_9BPC_EXT',0x96CC)
GL_SURFACE_COMPRESSION_FIXED_RATE_DEFAULT_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_DEFAULT_EXT',0x96C2)
GL_SURFACE_COMPRESSION_FIXED_RATE_NONE_EXT=_C('GL_SURFACE_COMPRESSION_FIXED_RATE_NONE_EXT',0x96C1)
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,arrays.GLintArray)
def glTexStorageAttribs2DEXT(target,levels,internalformat,width,height,attrib_list):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLsizei,_cs.GLenum,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,arrays.GLintArray)
def glTexStorageAttribs3DEXT(target,levels,internalformat,width,height,depth,attrib_list):pass



'''OpenGL extension ANGLE.program_binary

This module customises the behaviour of the 
OpenGL.raw.GLES2.ANGLE.program_binary to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension makes available a program binary format,
	PROGRAM_BINARY_ANGLE. It enables retrieving and loading of pre-linked
	ANGLE program objects.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/ANGLE/program_binary.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.ANGLE.program_binary import *
from OpenGL.raw.GLES2.ANGLE.program_binary import _EXTENSION_NAME

def glInitProgramBinaryANGLE():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Lists/tuples as data-format for storage

Note:
    This implementation is *far* less efficient than using Numpy
    to support lists/tuples, as the code here is all available in
    C-level code there.  This implementation is required to allow
    for usage without numpy installed.
"""

REGISTRY_NAME = 'lists'
import ctypes, _ctypes

# Note: these are the same definitions as for GLES, so we are not cross-polluting
from OpenGL.raw.GL import _types
from OpenGL.arrays import _arrayconstants as GL_1_1
from OpenGL import constant, error
from OpenGL._configflags import ERROR_ON_COPY
from OpenGL.arrays import formathandler
from OpenGL._bytes import bytes, unicode, as_8_bit

HANDLED_TYPES = (list, tuple)
import operator


def err_on_copy(func):
    """Decorator which raises informative error if we try to copy while ERROR_ON_COPY"""
    if not ERROR_ON_COPY:
        return func
    else:

        def raiseErrorOnCopy(self, value, *args, **named):
            raise error.CopyError(
                """%s passed, cannot copy with ERROR_ON_COPY set, please use an array type which has native data-pointer support (e.g. numpy or ctypes arrays)"""
                % (value.__class__.__name__,)
            )

        raiseErrorOnCopy.__name__ = getattr(func, '__name__', 'raiseErrorOnCopy')
        return raiseErrorOnCopy


class ListHandler(formathandler.FormatHandler):
    """Storage of array data in Python lists/arrays

    This mechanism, unlike multi-dimensional arrays, is not necessarily
    uniform in type or dimension, so we have to do a lot of extra checks
    to make sure that we get a correctly-structured array.  That, as
    well as the need to copy the arrays in Python code, makes this a far
    less efficient implementation than the numpy implementation, which
    does all the same things, but does them all in C code.

    Note: as an *output* format, this format handler produces ctypes
        arrays, not Python lists, this is done for convenience in coding
        the implementation, mostly.
    """

    @err_on_copy
    def from_param(self, instance, typeCode=None):
        try:
            return ctypes.byref(instance)
        except (TypeError, AttributeError) as err:
            array = self.asArray(instance, typeCode)
            pp = ctypes.c_void_p(ctypes.addressof(array))
            pp._temporary_array_ = (array,)
            return pp

    dataPointer = staticmethod(ctypes.addressof)
    HANDLED_TYPES = HANDLED_TYPES
    isOutput = True

    @err_on_copy
    @classmethod
    def voidDataPointer(cls, value):
        """Given value in a known data-pointer type, return void_p for pointer"""
        return ctypes.byref(value)

    @classmethod
    def zeros(cls, dims, typeCode):
        """Return array of zeros in given size"""
        type = GL_TYPE_TO_ARRAY_MAPPING[typeCode]
        for dim in dims:
            type *= dim
        return type()  # should expicitly set to 0s

    @classmethod
    def dimsOf(cls, x):
        """Calculate total dimension-set of the elements in x

        This is *extremely* messy, as it has to track nested arrays
        where the arrays could be different sizes on all sorts of
        levels...
        """
        try:
            _ = [len(x)]
        except (TypeError, AttributeError, ValueError) as err:
            return []
        else:
            childDimension = None
            for child in x:
                newDimension = cls.dimsOf(child)
                if childDimension is not None:
                    if newDimension != childDimension:
                        raise ValueError(
                            """Non-uniform array encountered: %s versus %s"""
                            % (
                                newDimension,
                                childDimension,
                            ),
                            x,
                        )

    @classmethod
    def arrayToGLType(cls, value):
        """Given a value, guess OpenGL type of the corresponding pointer"""

        result = ARRAY_TO_GL_TYPE_MAPPING.get(value._type_)
        if result is not None:
            return result
        raise TypeError(
            """Don't know GL type for array of type %r, known types: %s\nvalue:%s"""
            % (
                value._type_,
                list(ARRAY_TO_GL_TYPE_MAPPING.keys()),
                value,
            )
        )

    @classmethod
    def arraySize(cls, value, typeCode=None):
        """Given a data-value, calculate dimensions for the array"""
        dims = 1
        for base in cls.types(value):
            length = getattr(base, '_length_', None)
            if length is not None:
                dims *= length
        return dims

    @classmethod
    def types(cls, value):
        """Produce iterable producing all composite types"""
        dimObject = value
        while dimObject is not None:
            yield dimObject
            dimObject = getattr(dimObject, '_type_', None)
            if isinstance(dimObject, (bytes, unicode)):
                dimObject = None

    @classmethod
    def dims(cls, value):
        """Produce iterable of all dimensions"""
        for base in cls.types(value):
            length = getattr(base, '_length_', None)
            if length is not None:
                yield length

    @err_on_copy
    @classmethod
    def asArray(cls, value, typeCode=None):
        """Convert given value to a ctypes array value of given typeCode

        This does a *lot* of work just to get the data into the correct
        format.  It's not going to be anywhere near as fast as a numpy
        or similar approach!
        """
        if typeCode is None:
            raise NotImplementedError(
                """Haven't implemented type-inference for lists yet"""
            )
        arrayType = GL_TYPE_TO_ARRAY_MAPPING[typeCode]
        if isinstance(value, (list, tuple)):
            subItems = [cls.asArray(item, typeCode) for item in value]
            if subItems:
                for dim in cls.dimensions(subItems[0])[::-1]:
                    arrayType *= dim
                arrayType *= len(subItems)
                result = arrayType()
                result[:] = subItems
                return result
        else:
            return arrayType(value)

    @err_on_copy
    @classmethod
    def unitSize(cls, value, typeCode=None):
        """Determine unit size of an array (if possible)"""
        return tuple(cls.dims(value))[-1]

    @err_on_copy
    @classmethod
    def dimensions(cls, value, typeCode=None):
        """Determine dimensions of the passed array value (if possible)"""
        return tuple(cls.dims(value))

    @classmethod
    def arrayByteCount(cls, value, typeCode=None):
        """Given a data-value, calculate number of bytes required to represent"""
        return ctypes.sizeof(value)


ARRAY_TO_GL_TYPE_MAPPING = {
    _types.GLdouble: GL_1_1.GL_DOUBLE,
    _types.GLfloat: GL_1_1.GL_FLOAT,
    _types.GLint: GL_1_1.GL_INT,
    _types.GLuint: GL_1_1.GL_UNSIGNED_INT,
    _types.GLshort: GL_1_1.GL_SHORT,
    _types.GLushort: GL_1_1.GL_UNSIGNED_SHORT,
    _types.GLchar: GL_1_1.GL_CHAR,
    _types.GLbyte: GL_1_1.GL_BYTE,
    _types.GLubyte: GL_1_1.GL_UNSIGNED_BYTE,
}
GL_TYPE_TO_ARRAY_MAPPING = {
    GL_1_1.GL_DOUBLE: _types.GLdouble,
    GL_1_1.GL_FLOAT: _types.GLfloat,
    GL_1_1.GL_INT: _types.GLint,
    GL_1_1.GL_UNSIGNED_INT: _types.GLuint,
    GL_1_1.GL_SHORT: _types.GLshort,
    GL_1_1.GL_UNSIGNED_SHORT: _types.GLushort,
    GL_1_1.GL_CHAR: _types.GLchar,
    GL_1_1.GL_BYTE: _types.GLbyte,
    GL_1_1.GL_UNSIGNED_BYTE: _types.GLubyte,
    'f': _types.GLfloat,
    'd': _types.GLdouble,
    'i': _types.GLint,
    'I': _types.GLuint,
    'h': _types.GLshort,
    'H': _types.GLushort,
    'b': _types.GLbyte,
    'B': _types.GLubyte,
    's': _types.GLchar,
}



'''OpenGL extension VERSION.GL_4_6

This module customises the behaviour of the 
OpenGL.raw.GL.VERSION.GL_4_6 to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/VERSION/GL_4_6.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.VERSION.GL_4_6 import *
from OpenGL.raw.GL.VERSION.GL_4_6 import _EXTENSION_NAME

def glInitGl46VERSION():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glSpecializeShader.pConstantIndex size not checked against numSpecializationConstants
# INPUT glSpecializeShader.pConstantValue size not checked against numSpecializationConstants
glSpecializeShader=wrapper.wrapper(glSpecializeShader).setInputArraySize(
    'pConstantIndex', None
).setInputArraySize(
    'pConstantValue', None
)
### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES1 import _types as _cs
# End users want this...
from OpenGL.raw.GLES1._types import *
from OpenGL.raw.GLES1 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES1_OES_read_format'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES1,'GLES1_OES_read_format',error_checker=_errors._error_checker)
GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES=_C('GL_IMPLEMENTATION_COLOR_READ_FORMAT_OES',0x8B9B)
GL_IMPLEMENTATION_COLOR_READ_TYPE_OES=_C('GL_IMPLEMENTATION_COLOR_READ_TYPE_OES',0x8B9A)




'''OpenGL extension NV.stream_consumer_gltexture_yuv

This module customises the behaviour of the 
OpenGL.raw.EGL.NV.stream_consumer_gltexture_yuv to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/stream_consumer_gltexture_yuv.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.EGL import _types, _glgets
from OpenGL.raw.EGL.NV.stream_consumer_gltexture_yuv import *
from OpenGL.raw.EGL.NV.stream_consumer_gltexture_yuv import _EXTENSION_NAME

def glInitStreamConsumerGltextureYuvNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.multisampled_render_to_texture2

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.multisampled_render_to_texture2 to provide a more 
Python-friendly API

Overview (from the spec)
	
	The <attachment> parameters for FramebufferTexture2DMultisampleEXT is no
	longer required to be COLOR_ATTACHMENT0.  The attachment parameter now
	matches what is allowed in FramebufferTexture2D.  This means values like
	GL_COLOR_ATTACHMENTi, GL_DEPTH_ATTACHMENT, GL_STENCIL_ATTACHMENT, or
	GL_DEPTH_STENCIL_ATTACHMENT may be used.
	After the application has rendered into the mutisampled buffer, the application
	should be careful to not trigger an implicit flush by performing a client side
	read of the buffer (readpixels, copyteximage, blitframebuffer, etc) before any
	subsequent rendering which uses the contents of the buffer. This may cause the
	attachment to be downsampled before the following draw, which would potentially
	cause corruption.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/multisampled_render_to_texture2.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.multisampled_render_to_texture2 import *
from OpenGL.raw.GLES2.EXT.multisampled_render_to_texture2 import _EXTENSION_NAME

def glInitMultisampledRenderToTexture2EXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension QCOM.motion_estimation

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.motion_estimation to provide a more 
Python-friendly API

Overview (from the spec)
	
	Motion estimation, also referred to as optical flow, is the process of
	producing motion vectors that convey the 2D transformation from a reference
	image to a target image.  There are various uses of motion estimation, such as
	frame extrapolation, compression, object tracking, etc.
	
	This extension adds support for motion estimation in OpenGL ES by adding
	functions which take the reference and target images and populate an
	output texture containing the corresponding motion vectors.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/motion_estimation.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.motion_estimation import *
from OpenGL.raw.GLES2.QCOM.motion_estimation import _EXTENSION_NAME

def glInitMotionEstimationQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_NV_memory_object_sparse'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_NV_memory_object_sparse',error_checker=_errors._error_checker)

@_f
@_p.types(None,_cs.GLenum,_cs.GLintptr,_cs.GLsizeiptr,_cs.GLuint,_cs.GLuint64,_cs.GLboolean)
def glBufferPageCommitmentMemNV(target,offset,size,memory,memOffset,commit):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLintptr,_cs.GLsizeiptr,_cs.GLuint,_cs.GLuint64,_cs.GLboolean)
def glNamedBufferPageCommitmentMemNV(buffer,offset,size,memory,memOffset,commit):pass
@_f
@_p.types(None,_cs.GLenum,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLuint,_cs.GLuint64,_cs.GLboolean)
def glTexPageCommitmentMemNV(target,layer,level,xoffset,yoffset,zoffset,width,height,depth,memory,offset,commit):pass
@_f
@_p.types(None,_cs.GLuint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLint,_cs.GLsizei,_cs.GLsizei,_cs.GLsizei,_cs.GLuint,_cs.GLuint64,_cs.GLboolean)
def glTexturePageCommitmentMemNV(texture,layer,level,xoffset,yoffset,zoffset,width,height,depth,memory,offset,commit):pass



'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GL import _types as _cs
# End users want this...
from OpenGL.raw.GL._types import *
from OpenGL.raw.GL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GL_HP_convolution_border_modes'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GL,'GL_HP_convolution_border_modes',error_checker=_errors._error_checker)
GL_CONSTANT_BORDER_HP=_C('GL_CONSTANT_BORDER_HP',0x8151)
GL_CONVOLUTION_BORDER_COLOR_HP=_C('GL_CONVOLUTION_BORDER_COLOR_HP',0x8154)
GL_IGNORE_BORDER_HP=_C('GL_IGNORE_BORDER_HP',0x8150)
GL_REPLICATE_BORDER_HP=_C('GL_REPLICATE_BORDER_HP',0x8153)




'''OpenGL extension EXT.framebuffer_blit_layers

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.framebuffer_blit_layers to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extensions defines the behaviour for copying data from one layered
	framebuffer to another layered framebuffer.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/framebuffer_blit_layers.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.framebuffer_blit_layers import *
from OpenGL.raw.GLES2.EXT.framebuffer_blit_layers import _EXTENSION_NAME

def glInitFramebufferBlitLayersEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.fragment_shading_rate

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.fragment_shading_rate to provide a more 
Python-friendly API

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/fragment_shading_rate.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.fragment_shading_rate import *
from OpenGL.raw.GLES2.EXT.fragment_shading_rate import _EXTENSION_NAME

def glInitFragmentShadingRateEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )

# INPUT glGetFragmentShadingRatesEXT.shadingRates size not checked against maxCount
glGetFragmentShadingRatesEXT=wrapper.wrapper(glGetFragmentShadingRatesEXT).setInputArraySize(
    'count', 1
).setInputArraySize(
    'shadingRates', None
)
### END AUTOGENERATED SECTION


'''OpenGL extension NV.generate_mipmap_sRGB

This module customises the behaviour of the 
OpenGL.raw.GLES2.NV.generate_mipmap_sRGB to provide a more 
Python-friendly API

Overview (from the spec)
	
	EXT_sRGB requires GenerateMipmap() to throw INVALID_OPERATION on textures
	with sRGB encoding. NV_generate_mipmap_sRGB lifts this restriction.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/generate_mipmap_sRGB.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.NV.generate_mipmap_sRGB import *
from OpenGL.raw.GLES2.NV.generate_mipmap_sRGB import _EXTENSION_NAME

def glInitGenerateMipmapSrgbNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.WGL import _types as _cs
# End users want this...
from OpenGL.raw.WGL._types import *
from OpenGL.raw.WGL import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'WGL_EXT_create_context_es2_profile'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.WGL,'WGL_EXT_create_context_es2_profile',error_checker=_errors._error_checker)
WGL_CONTEXT_ES2_PROFILE_BIT_EXT=_C('WGL_CONTEXT_ES2_PROFILE_BIT_EXT',0x00000004)




'''Autogenerated by xml_generate script, do not edit!'''
from OpenGL import platform as _p, arrays
# Code generation uses this
from OpenGL.raw.GLES2 import _types as _cs
# End users want this...
from OpenGL.raw.GLES2._types import *
from OpenGL.raw.GLES2 import _errors
from OpenGL.constant import Constant as _C

import ctypes
_EXTENSION_NAME = 'GLES2_QCOM_shading_rate'
def _f( function ):
    return _p.createFunction( function,_p.PLATFORM.GLES2,'GLES2_QCOM_shading_rate',error_checker=_errors._error_checker)
GL_SHADING_RATE_1X1_PIXELS_QCOM=_C('GL_SHADING_RATE_1X1_PIXELS_QCOM',0x96A6)
GL_SHADING_RATE_1X2_PIXELS_QCOM=_C('GL_SHADING_RATE_1X2_PIXELS_QCOM',0x96A7)
GL_SHADING_RATE_2X1_PIXELS_QCOM=_C('GL_SHADING_RATE_2X1_PIXELS_QCOM',0x96A8)
GL_SHADING_RATE_2X2_PIXELS_QCOM=_C('GL_SHADING_RATE_2X2_PIXELS_QCOM',0x96A9)
GL_SHADING_RATE_4X2_PIXELS_QCOM=_C('GL_SHADING_RATE_4X2_PIXELS_QCOM',0x96AC)
GL_SHADING_RATE_4X4_PIXELS_QCOM=_C('GL_SHADING_RATE_4X4_PIXELS_QCOM',0x96AE)
GL_SHADING_RATE_PRESERVE_ASPECT_RATIO_QCOM=_C('GL_SHADING_RATE_PRESERVE_ASPECT_RATIO_QCOM',0x96A5)
GL_SHADING_RATE_QCOM=_C('GL_SHADING_RATE_QCOM',0x96A4)
@_f
@_p.types(None,_cs.GLenum)
def glShadingRateQCOM(rate):pass



#! /usr/bin/env python
"""Python 3.x buffer-handling (currently just for bytes/bytearray types)
"""
import ctypes, sys

if sys.version_info[:2] < (2, 6):
    raise ImportError('Buffer interface only usable on Python 2.6+')
from ._arrayconstants import *

PyBUF_SIMPLE = 0
PyBUF_WRITABLE = PyBUF_WRITEABLE = 0x0001
PyBUF_ND = 0x0008
PyBUF_STRIDES = 0x0010 | PyBUF_ND
PyBUF_CONTIG = PyBUF_ND | PyBUF_WRITABLE
PyBUF_CONTIG_RO = PyBUF_ND
PyBUF_C_CONTIGUOUS = 0x0020 | PyBUF_STRIDES
PyBUF_FORMAT = 0x0004

# Python 2.6 doesn't define this...
c_ssize_t = getattr(ctypes, 'c_ssize_t', ctypes.c_ulong)

_fields_ = [
    ('buf', ctypes.c_void_p),
    ('obj', ctypes.c_void_p),
    ('len', c_ssize_t),
    ('itemsize', c_ssize_t),
    ('readonly', ctypes.c_int),
    ('ndim', ctypes.c_int),
    ('format', ctypes.c_char_p),
    ('shape', ctypes.POINTER(c_ssize_t)),
    ('strides', ctypes.POINTER(c_ssize_t)),
    ('suboffsets', ctypes.POINTER(c_ssize_t)),
]


if sys.version_info[:2] <= (2, 6) or sys.version_info[:2] >= (3, 3):
    # Original structure was eventually restored in 3.3, so just
    # 2.7 through 3.2 uses the "enhanced" structure below
    _fields_.extend(
        [
            ('internal', ctypes.c_void_p),
        ]
    )
else:
    # Sigh, this structure seems to have changed with Python 3.x...
    _fields_.extend(
        [
            ('smalltable', ctypes.c_size_t * 2),
            ('internal', ctypes.c_void_p),
        ]
    )


class Py_buffer(ctypes.Structure):
    """Wrapper around the Python buffer structure..."""

    @classmethod
    def from_object(
        cls, object, flags=PyBUF_STRIDES | PyBUF_FORMAT | PyBUF_C_CONTIGUOUS
    ):
        """Create a new Py_buffer referencing ram of object"""
        if not CheckBuffer(object):
            raise TypeError(
                "%s type does not support Buffer Protocol" % (object.__class__,)
            )
        buf = cls()
        # deallocation of the buf causes glibc abort :(
        result = GetBuffer(object, buf, flags)
        if result != 0:
            raise ValueError("Unable to retrieve Buffer from %s" % (object,))
        if not buf.buf:
            raise ValueError("Null pointer result from %s" % (object,))
        return buf

    _fields_ = _fields_

    @property
    def dims(self):
        return self.shape[: self.ndim]

    def __len__(self):
        return self.shape[0]

    @property
    def dim_strides(self):
        if self.strides:
            return self.strides[: self.ndim]
        return None

    def __enter__(self):
        pass

    def __exit__(self, exc_type=None, exc_value=None, traceback=None):
        if self.obj:
            ReleaseBuffer(self)

    def __del__(self):
        if self.obj:
            ReleaseBuffer(self)


BUFFER_POINTER = ctypes.POINTER(Py_buffer)


try:
    CheckBuffer = ctypes.pythonapi.PyObject_CheckBuffer
    CheckBuffer.argtypes = [ctypes.py_object]
    CheckBuffer.restype = ctypes.c_int
except AttributeError as err:
    # Python 2.6 doesn't appear to have CheckBuffer support...
    def CheckBuffer(x):
        True


IncRef = ctypes.pythonapi.Py_IncRef
IncRef.argtypes = [ctypes.py_object]

GetBuffer = ctypes.pythonapi.PyObject_GetBuffer
GetBuffer.argtypes = [ctypes.py_object, BUFFER_POINTER, ctypes.c_int]
GetBuffer.restype = ctypes.c_int

ReleaseBuffer = ctypes.pythonapi.PyBuffer_Release
ReleaseBuffer.argtypes = [BUFFER_POINTER]
ReleaseBuffer.restype = None



'''OpenGL extension QCOM.texture_lod_bias

This module customises the behaviour of the 
OpenGL.raw.GLES2.QCOM.texture_lod_bias to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension reintroduces the texture LOD bias operation from
	EXT_texture_lod_bias which is written based on OpenGL ES 1.1.
	It provides a mean to bias the lambda computation by a
	constant (signed) value. This bias can provide a way to blur or
	pseudo-sharpen OpenGL ES's standard texture filtering.
	
	This blurring or pseudo-sharpening may be useful for special
	effects (such as depth-of-field effects) or image processing
	techniques (where the mipmap levels act as pre-downsampled image
	versions). On some implementations, increasing the texture LOD
	bias may improve texture filtering performance (at the cost of
	texture blurriness).

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/QCOM/texture_lod_bias.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.QCOM.texture_lod_bias import *
from OpenGL.raw.GLES2.QCOM.texture_lod_bias import _EXTENSION_NAME

def glInitTextureLodBiasQCOM():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


'''OpenGL extension EXT.texture_type_2_10_10_10_REV

This module customises the behaviour of the 
OpenGL.raw.GLES2.EXT.texture_type_2_10_10_10_REV to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension adds a new texture data type, unsigned 2.10.10.10 ABGR,
	which can be used with RGB or RGBA formats.

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/EXT/texture_type_2_10_10_10_REV.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GLES2 import _types, _glgets
from OpenGL.raw.GLES2.EXT.texture_type_2_10_10_10_REV import *
from OpenGL.raw.GLES2.EXT.texture_type_2_10_10_10_REV import _EXTENSION_NAME

def glInitTextureType2101010RevEXT():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION


"""Common code for accelerated modules"""

import logging
from OpenGL import _configflags

needed_version = (3, 1, 9)
_log = logging.getLogger("OpenGL.acceleratesupport")
try:
    import OpenGL_accelerate

    if _configflags.USE_ACCELERATE:
        if OpenGL_accelerate.__version_tuple__ < needed_version:
            _log.warning(
                """Incompatible version of OpenGL_accelerate found, need at least %s found %s""",
                needed_version,
                OpenGL_accelerate.__version_tuple__,
            )
            raise ImportError("""Old version of OpenGL_accelerate""")
        ACCELERATE_AVAILABLE = True
        _log.debug("""OpenGL_accelerate module loaded""")
    else:
        raise ImportError("""Acceleration disabled""")
except ImportError as err:
    _log.info("""No OpenGL_accelerate module loaded: %s""", err)
    ACCELERATE_AVAILABLE = False



'''OpenGL extension NV.primitive_shading_rate

This module customises the behaviour of the 
OpenGL.raw.GL.NV.primitive_shading_rate to provide a more 
Python-friendly API

Overview (from the spec)
	
	This extension builds on top of the NV_shading_rate_image extension to
	provide OpenGL API support for using a per-primitive shading rate value to
	control the computation of the rate used to process each fragment.
	
	In the NV_shading_rate_image extension, the shading rate for each fragment
	produced by a primitive is determined by looking up a texel in the bound
	shading rate image and using that value as an index into a shading rate
	palette.  That extension provides a separate shading rate image lookup
	enable and palette for each viewport.  When a primitive is rasterized, the
	implementation uses the enable and palette associated with the primitive's
	viewport to determine the shading rate.
	
	This extension decouples the shading rate image enables and palettes from
	viewports.  The number of enables/palettes now comes from the
	implementation-dependent constant SHADING_RATE_IMAGE_PALETTE_COUNT_NV.  When
	SHADING_RATE_IMAGE_PER_PRIMITIVE_NV (added here) is enabled, the value of
	the new gl_ShadingRateNV built-in output is used to select an enable and
	palette to determine the shading rate.  Otherwise, the viewport number for
	the primitive is used, as in NV_shading_rate_image.
	

The official definition of this extension is available here:
http://www.opengl.org/registry/specs/NV/primitive_shading_rate.txt
'''
from OpenGL import platform, constant, arrays
from OpenGL import extensions, wrapper
import ctypes
from OpenGL.raw.GL import _types, _glgets
from OpenGL.raw.GL.NV.primitive_shading_rate import *
from OpenGL.raw.GL.NV.primitive_shading_rate import _EXTENSION_NAME

def glInitPrimitiveShadingRateNV():
    '''Return boolean indicating whether this extension is available'''
    from OpenGL import extensions
    return extensions.hasGLExtension( _EXTENSION_NAME )


### END AUTOGENERATED SECTION